/**********************************************************************
 * tcp_socket.h                                           November 1999
 * Horms                                             horms@vergenet.net
 *
 * Open a tcp socket to a server (we are the client)
 *
 * Released under the GNU LGPL
 *
 **********************************************************************/

#ifndef TCP_PIPE_NYM
#define TCP_PIPE_NYM

#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>
#include <arpa/inet.h>


/**********************************************************************
 * tcp_socket_client_open
 * Open a socket connection as a client
 * pre: host: hostname or ipaddress to open socket to
 *      port: name or number to open
 * post: socket is opened
 * return: open socket
 *         -1 on error
 **********************************************************************/

int tcp_socket_client_open(char *host, char *port);


/**********************************************************************
 * tcp_socket_port_portno
 * port number of a service given as a string either
 * the port number or the service name as per /etc/services
 * pre: port name as per /etc/services or port number as a string
 * return: port number
 *         0 on error or if port name cannot be found in /etc/services
 **********************************************************************/

unsigned short int tcp_socket_port_portno(const char *port);


/**********************************************************************
 * tcp_socket_port_str_is_number
 * Test if a null terminated string is composed entirely of digits (0-9)
 * pre: String
 * return: 1 if string contains only digits and null teminator
 *         0 otherwise
 **********************************************************************/


int tcp_socket_port_str_is_number(const char *str);


/**********************************************************************
 * tcp_socket_pipe
 * pipe data between two file discriptors until there is an error,
 * timeout or one or both the file descriptors are closed.
 * pre: fd_a_in:  one of the file descriptors to read from
 *      fd_a_out: one of the file descriptors to write to
 *      fd_b_in:  one of the file descriptors to read from
 *      fd_b_out: one of the file descriptors to write to
 *      buffer:   allocated buffer to read data into
 *      buffer_length: size of buffer in bytes
 *      idle_timeout:  timeout in seconds to wait for input
 *                     timout of 0 = infinite timeout
 *      return_a_read_bytes: Pointer to int where number
 *                           of bytes read from a will be recorded.
 *      return_b_read_bytes: Pointer to int where number
 *                           of bytes read from b will be recorded.
 * post: data is read from fd_a_in and written to fd_b_out and
 *       data is read from fd_b_in and written to fd_a_out.
 * return: -1 on error
 *         1 on idle timeout
 *         0 otherwise (one of fd_a_in or fd_b_in closes gracefully)
 **********************************************************************/

int tcp_socket_pipe(
  int fd_a_in, 
  int fd_a_out, 
  int fd_b_in, 
  int fd_b_out, 
  unsigned char *buffer, 
  int buffer_length,
  int timeout,
  int *return_a_read_bytes,
  int *return_b_read_bytes
);


/**********************************************************************
 * tcp_socket_pipe_read_write
 * Read data from one file descriptor (socket) and write it to another
 * pre: in_fd: file descriptor to read from
 *      out_fd: file descriptor to write to
 *      buffer: allocated buffer to store read data in
 *      buffer_length: size of the buffer
 * post: at most buffer_length bytes are read from in_fd and written 
 *       to out_fd. 
 * return: bytes read on sucess
 *         0 on EOF
 *         -1 on error
 **********************************************************************/

int tcp_socket_pipe_read_write(
  int in_fd, 
  int out_fd, 
  unsigned char *buffer,
  int buffer_length
);


/**********************************************************************
 * tcp_socket_pipe_write_bytes
 * write a n bytes of a buffer to fd
 * Pre: fd: File descriptor to write to
 *      buffer: buffer to write
 *      n: number or bytes to write
 * Return: -1 on error
 *         0 otherwise
 **********************************************************************/

int tcp_socket_pipe_write_bytes(
  const int fd, 
  const unsigned char *buffer, 
  const ssize_t n
);


/**********************************************************************
 * tcp_socker_server_connect
 * Listen on a tcp port for incoming client connections 
 * When one is recieved fork
 * In the Child: close the listening file desctiptor
 *               return the file descriptor that is 
 *               a socket connection to the client
 * In the Server: close the socket to the client and loop
 * pre: port: port to listen to, an ascii representation of a number
 *            or an entry from /etc/services
 *      interface_address: If NULL bind to all interfaces, else
 *                         bind to interface(es) with this address.
 *      maximum_connections: maximum number of active connections
 *      to handle. If 0 then an number of connections is unlimited.
 *      return_from: pointer to a struct_in addr where the 
 *                   connecting client's IP address will
 *                   be placed. Ignored if NULL
 *      return_to: pointer to a in addr where the IP address the 
 *                 server accepted the connection on will be placed.
 *                 Ignored if NULL
 * post: Client sockets are returned in child processes
 *       In the parent process the function doesn't exit, other 
 *       than on error.
 *       if return_from is non-null, it is seeded with cleints address
 * return: open client socket, if connection is accepted.
 *         -1 on error
 **********************************************************************/

int tcp_socket_server_connect(
  const char *port,
  const char *interface_address,
  const unsigned int maximum_connections,
  struct sockaddr_in *return_from,
  struct sockaddr_in *return_to
);


/**********************************************************************
 * tcp_socker_server_reaper
 * A signal handler that waits for SIGCHLD and runs wait3 to free
 * the resources of any exited children. This stops zombie processes
 * from hanging around.
 * pre: SIGCHLD is recieved by the process
 * post: Resoerces of any exited children are freed
 * return: 0
 **********************************************************************/

void tcp_socket_server_reaper(void);

#endif
